#include "ShaderConstants.fxh"
#include "Util.fxh"

struct PS_Input
{
	float4 position : SV_Position;
	
	float3 xyz : XYZ_POS;
	
	float3 cloud : CLOUD;
	
	float3 screen_pos : SCREEN_POS;
	
	float new_fog : NEW_FOG;
	
	float sun : SUN_P;
#ifndef BYPASS_PIXEL_SHADER
	lpfloat4 color : COLOR;
	snorm float2 uv0 : TEXCOORD_0_FB_MSAA;
	snorm float2 uv1 : TEXCOORD_1_FB_MSAA;
#endif

#ifdef NEAR_WATER
	float cameraDist : TEXCOORD_2;
#endif

#ifdef FOG
	float4 fogColor : FOG_COLOR;
#endif
};

struct PS_Output
{
	float4 color : SV_Target;
};
// =========== NOISE SET ============
float hash(float2 p){
    p  = frac(p * .1031);
    p += dot(p, p.yx + 19.19);
    return frac((p.x + p.y) * p.x);
}
float4 noise( float2 p )
{
    float2 i = floor( p );
    float2 f = frac( p );
	
	float2 u = f*f*(3.0-2.0*f);

    return lerp( lerp( hash( i + float2(0.0,0.0) ), 
                     hash( i + float2(1.0,0.0) ), u.x),
                lerp( hash( i + float2(0.0,1.0) ), 
                     hash( i + float2(1.0,1.0) ), u.x), u.y);
}
float4 watercolorH(float2 x) {
    float2 p = floor(x);
    float2 f = frac(x);

	float a = hash(p);
    float b = hash(p + float2(1., 0.));
    float c = hash(p + float2(0., 1.));
    float d = hash(p + float2(1., 1.));
    
    float2 g = f - 1.;
    g *= g;
    
    float2 v = 1. - g;
    v *= v;
	
	float noise = lerp(lerp(a, b, v.x), lerp(c, d, v.x), v.y);
    
    float2 z = 30.*g*f*f;
	
    float2 derivatives = z * (float2(b, c) + (a - b - c + d) * v.yx - a);
    
    float3 normal = cross(float3(1,0,derivatives.x),float3(0,1,derivatives.y));

	return float4(normal, noise);
}
// =========== CLOUD SET ============
float4 cloudH( float2 x ) {
    float f = 0.0;
	
	float h = 0.0;
    for (float i=1.0;i<10.0;i++) {
        h +=noise(x*pow(2.6, i)) * 0.9 * pow(0.425, i);
    }
		f = 0.5000*noise( x ); 
		x = x*pow(0.1, i);
        f += 0.2500*noise( x ); 
		x = x*pow(0.31, i);
        f += 0.1250*noise( x ); 
		x = x*pow(0.32, i);
        f += 0.0625*noise( x );
    float n = f + h;
	
	return n;
}
// =========== WATER SET ============
float waterH(float3 posxz) {
    float wave = 0.0;
	float wave2 = 0.0;
	
    for (float i=1.0;i<10.0;i++) {
		wave += noise(posxz.xz * pow(1.6, i)) * 0.9 * pow(0.6, i);
		wave2 += noise(posxz.xz * pow(1.6, i)) * 0.9 * pow(0.4, i);
    }
    return wave*wave2;
}

// =========== WORLDLIGHT SET ============
float3 tonemap(float3 clr) {

	float gamma = 1.0;
	float exposure = 1.05;
	float contrast = 0.04;
    float saturation = 1.05;
	
	clr = pow(clr, float3(gamma,gamma,gamma));
	clr *= exposure;
	clr = max(clr - contrast, 0.0);

    float luma = dot(clr, float3(1.0,1.0,1.0));
    float3 chroma = clr - luma;
    clr = (chroma * saturation) + luma;
 
	return clr;

}
// =========== MASK SET ============
bool Nether(float4 Fog){
if((Fog.r>Fog.b&&Fog.r>Fog.g)&&(Fog.r<0.5&&Fog.b<0.1&&Fog.g<0.1)){
return true;
}else{
return false;
}}
void main( in PS_Input PSInput, out PS_Output PSOutput )
{
#ifdef BYPASS_PIXEL_SHADER
    PSOutput.color = float4(0.0f, 0.0f, 0.0f, 0.0f);
    return;
#else

#if !defined(TEXEL_AA) || !defined(TEXEL_AA_FEATURE) || (VERSION < 0xa000 /*D3D_FEATURE_LEVEL_10_0*/) 
	float4 diffuse = TEXTURE_0.Sample(TextureSampler0, PSInput.uv0);
#else
	float4 diffuse = texture2D_AA(TEXTURE_0, TextureSampler0, PSInput.uv0 );
#endif

#ifdef SEASONS_FAR
	diffuse.a = 1.0f;
	PSInput.color.b = 1.0f;
#endif

#ifdef ALPHA_TEST
//If we know that all the verts in a triangle will have the same alpha, we should cull there first.
	#ifdef ALPHA_TO_COVERAGE
		float alphaThreshold = .05f;
	#else
		float alphaThreshold = .5f;
	#endif
	if(diffuse.a < alphaThreshold)
		discard;
#endif

#if !defined(ALWAYS_LIT)
    float4 Dark = TEXTURE_1.Sample(TextureSampler1, PSInput.uv1);
	diffuse = diffuse * Dark;
#endif

#ifndef SEASONS

#if !defined(ALPHA_TEST) && !defined(BLEND)
	diffuse.a = PSInput.color.a;
#elif defined(BLEND)
	diffuse.a *= PSInput.color.a;

#ifdef NEAR_WATER
//Enjoy
#endif

#endif	
	diffuse.rgb *= PSInput.color.rgb;
#else
	float2 uvv = PSInput.color.xy;
	diffuse.rgb *= lerp(1.0f, TEXTURE_2.Sample(TextureSampler2, uvv).rgb*2.0f, PSInput.color.b);
	diffuse.rgb *= PSInput.color.aaa;
	diffuse.a = 1.0f;
#endif
// =========== SET ============
float FogOut = saturate(PSInput.new_fog);

float SnowFlag =0.0;

float WaterFlag =0.0;

float4 Fog = FOG_COLOR;

float Night = Fog.r+Fog.g+Fog.b;

float sha = 1.0;
//Color set

float4 TorchColor = float4(5.0,1.5,-5.0,0.25);

float3 FogColor = float3(0.2,0.2,0.2);
if(Night<0.6){
FogColor = float3(0.175,0.2075,0.34);
}
float4 WaterColor = lerp(float4(0.5, 0.5, .5, 0.65), float4(0.5, 0.5, .5, 0.65), Fog.a);
if(PSInput.uv1.y<0.9){
WaterColor = lerp(float4(0.3, 0.3, .3, 0.5), float4(0.3, 0.3, .3, 0.5), Fog.a);}

float4 CloudColor = Fog;

CloudColor.a = 0.5;

//Pos set
float2 cloud = PSInput.cloud.xz/25.0;

cloud.x += TIME*0.075;

float3 water = PSInput.xyz ;

water.x += TIME*0.25;
water.x += (water.z - TIME)*0.05;
water.z -= (water.x + TIME*0.1)*1.0;

float deltaPos = 0.4;
// =========== WATER ============
	float h0 = waterH(water);
	float h1 = waterH(water + float3(deltaPos,0.0,0.0)*float3(TIME, TIME, TIME));
	float h2 = waterH(water + float3(-deltaPos,0.0,0.0)*float3(TIME, TIME, TIME));
	float h3 = waterH(water + float3(0.0,0.0,deltaPos)*float3(TIME, TIME, TIME));
	float h4 = waterH(water + float3(0.0,0.0,-deltaPos)*float3(TIME, TIME, TIME));

	float xDelta = ((h1-h0)+(h0-h2))/deltaPos;
	float yDelta = ((h3-h0)+(h0-h4))/deltaPos;
	float3 watermap = normalize(float3(xDelta,yDelta,1.0-xDelta*xDelta-yDelta*yDelta));

    //float view_angle = acos(abs(cos_between_vecs(PSInput.look_vector, water_normal)));
	
	float4 waterbump = watercolorH(float4 (watermap,Fog.a));
    
	float wbump = dot(diffuse , waterbump);
	
	WaterColor *= WaterColor * wbump;
	
	CloudColor *= cloudH(cloud);
	
#ifdef NEAR_WATER
    sha = 0.0;
    WaterFlag = 1.0;
	diffuse = lerp(float4(0.4,0.4,0.4,0.5), float4(0.4,0.4,0.4,0.5) , Fog.a);
	diffuse *= diffuse + CloudColor + WaterColor ;
#endif
//Nether
if(Nether(Fog)){
diffuse.r *= 1.2;
FogColor = float3(0.3,0.1,0.05);
}

// Fog Set

diffuse.rgb = diffuse.rgb + lerp(diffuse.rgb*0.45,FogColor,min(max(FogOut*0.4 ,0.0),1.0));

diffuse.rgb = lerp(diffuse.rgb, diffuse.rgb+FogOut*FogColor, Fog);

//Light
diffuse.rgb = tonemap(diffuse.rgb);

//Rain
if(Night>0.6){
if(FOG_CONTROL.x < 0.55 && FOG_CONTROL.x > 0.1){
#ifdef NEAR_WATER
diffuse.a *= 0.55;
diffuse *= diffuse + CloudColor + WaterColor;
#endif
}}

// =========== TORCH ============

float4 TorchFog =Fog;

float4 TorchPos = PSInput.uv1.x;

TorchFog *= TorchFog.r*0.6;

float TorchFogSet = TorchFog.r*0.6;

diffuse += (TorchColor)*pow(TorchPos.x*0.8,5.0)*TorchPos.y*(1.5-TorchFogSet)*0.5;
// =========== DOF ============
 
float4 Dofc = float4(0.2,0.2,0.2,0.2);

float center_dist = min(1.5, length(PSInput.screen_pos.xy*0.6));
float edge_effect = max(0.0, (center_dist - 0.3) * 3.7);
float3 scrli = float3(max(150.0, 300.0 - edge_effect * 110.0),max(150.0, 300.0 - edge_effect * 110.0),max(150.0, 300.0 - edge_effect * 110.0))*0.0035;
diffuse.rgb *= max(scrli,0.1);
diffuse.rgb *= scrli;

#ifdef FOG
	diffuse.rgb = (lerp( diffuse.rgb, diffuse.rgb, diffuse.a ));
if(((PSInput.color.r*1.5<=PSInput.color.b)&&(PSInput.color.b>=PSInput.color.g*1.05))&&WaterFlag!=1.0&&PSInput.color.a>=0.0&&SnowFlag!=1.0){
//by Creeper
	diffuse.rgb = tonemap(lerp( diffuse.rgb, diffuse.rgb, diffuse.a ));
	diffuse = lerp(float4(0.4,0.4,0.4,0.5), float4(0.4,0.4,0.4,0.5) , Fog.a);
	diffuse *= diffuse + CloudColor + WaterColor;
}
#endif

	PSOutput.color = diffuse;

#ifdef VR_MODE
	// On Rift, the transition from 0 brightness to the lowest 8 bit value is abrupt, so clamp to 
	// the lowest 8 bit value.
	PSOutput.color = max(PSOutput.color, 1 / 255.0f);
#endif

#endif // BYPASS_PIXEL_SHADER
}